package se.su.chordDiagram;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;

import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.socialnetwork.SNEdge;
import org.processmining.models.graphbased.directed.socialnetwork.SNNode;
import org.processmining.models.graphbased.directed.socialnetwork.SocialNetwork;

@Plugin(name = "Annotated Chord export", returnLabels = {}, returnTypes = {}, parameterLabels = {
		"Social Network", "File" }, userAccessible = true)
@UIExportPlugin(description = "Annotated Chord html files", extension = "html")
public class AnnotatedChordExportPlugin {

	@PluginVariant(variantLabel = "Annotated Chord export (Social Network)", requiredParameterLabels = { 0, 1 })
	public void export(PluginContext context, SocialNetwork sn, File file) throws IOException {
		exportToFile(sn, file);
	}
	
	public void exportToFile(SocialNetwork sn, File file) throws IOException {
		FileWriter writer = new FileWriter(file);
		writer.write(getHTML(sn));
		writer.close();
	} 
	
	public String getHTML(SocialNetwork sn){
		StringBuilder htmlBuilder = new StringBuilder();
		
		htmlBuilder.append("<!DOCTYPE html>\n");
		htmlBuilder.append("<html>\n");
		htmlBuilder.append("<head>\n");
		htmlBuilder.append("	<meta charset=\"utf-8\">\n");
		htmlBuilder.append("	<title>Chord Diagram</title>\n");
		htmlBuilder.append("	<!-- This diagram is developed based on Andrew's Chord Example http://bl.ocks.org/AndrewRP/7468330 -->\n");
		htmlBuilder.append("	<style>\n");
		htmlBuilder.append("	body { background: #fcfcfa; color: #333; font-family: \"PT Serif\", serif; margin: 1em auto 4em auto; position: relative; width: 960px; }\n");
		htmlBuilder.append("	svg { font: 10px sans-serif; }\n");
		htmlBuilder.append("	#circle circle { fill: none; pointer-events: all; }\n");
		htmlBuilder.append("	path.chord { stroke: #000; stroke-width: .25px;}\n");
		htmlBuilder.append("	#circle:hover path.fade { display: none; }\n");
		htmlBuilder.append("	</style>\n");
		htmlBuilder.append("	<script src=\"http://d3js.org/d3.v2.min.js?2.8.1\"></script>\n");
		htmlBuilder.append("</head>\n");
		htmlBuilder.append("<body>\n");
		htmlBuilder.append("<p></aside>\n");
		htmlBuilder.append("<script>\n");
		htmlBuilder.append("		var width = 720, height = 720, outerRadius = Math.min(width, height) / 2 - 10, innerRadius = outerRadius - 24;\n");
		htmlBuilder.append("		var formatPercent = d3.format(\".1%\");\n");
		htmlBuilder.append("		var arc = d3.svg.arc().innerRadius(innerRadius).outerRadius(outerRadius);\n");
		htmlBuilder.append("		var layout = d3.layout.chord().padding(.04).sortSubgroups(d3.descending).sortChords(d3.ascending);\n");
		htmlBuilder.append("		var path = d3.svg.chord().radius(innerRadius);\n");
		htmlBuilder.append("		var svg = d3.select(\"body\").append(\"svg\").attr(\"width\", width).attr(\"height\", height).append(\"g\").attr(\"id\", \"circle\").attr(\"transform\", \"translate(\" + width / 2 + \",\" + height / 2 + \")\");\n");
		htmlBuilder.append("		svg.append(\"circle\").attr(\"r\", outerRadius);\n");
		htmlBuilder.append("		 \n");
		
		
		htmlBuilder.append(getSocialNetworkMatrix(sn));
		
		htmlBuilder.append("\n");
		
		htmlBuilder.append(getSocialNetworkNodes(sn));
		
		
		htmlBuilder.append("		var fill = d3.scale.ordinal().domain(d3.range(Chordnodes)).range([\"#FF9999\", \"#99FF99\", \"#9999FF\", \"#AA5555\", \"#55AA55\", \"#5555AA\", \"#11AACC\", \"#CCAA11\", \"#AA11CC\", \"#AACC11\", \"#CC11AA\", \"#11CCAA\", \"#FF8877\", \"#8877FF\"]);\n");
		htmlBuilder.append("		layout.matrix(matrix);\n");
		htmlBuilder.append("		var group = svg.selectAll(\".group\").data(layout.groups).enter().append(\"g\").attr(\"class\", \"group\").on(\"mouseover\", mouseover);\n");
		htmlBuilder.append("		var groupPath = group.append(\"path\").attr(\"id\", function(d, i) { return \"group\" + i; }).attr(\"d\", arc).style(\"fill\", function(d) { return fill(d.index); });\n");
		htmlBuilder.append("		// Add a text label.\n");
		htmlBuilder.append("		var groupText = group.append(\"text\").attr(\"x\", 6).attr(\"dy\", 15);\n");
		htmlBuilder.append("		groupText.append(\"textPath\").attr(\"xlink:href\", function(d, i) { return \"#group\" + i; }).text(function(d, i) { return Chordnodes[i]; });\n");
		htmlBuilder.append("		// Remove the labels that don't fit. :(\n");
		htmlBuilder.append("		groupText.filter(function(d, i) { return groupPath[0][i].getTotalLength() / 2 - 16 < this.getComputedTextLength(); }).remove();\n");
		htmlBuilder.append("		// Add the chords.\n");
		htmlBuilder.append("		var chord = svg.selectAll(\".chord\").data(layout.chords).enter().append(\"path\").attr(\"class\", \"chord\").style(\"fill\", function(d) { var chColor = \"#EEEEEE\"; if(d.source.value>d.target.value) chColor = fill(d.source.index); if(d.source.value<d.target.value) chColor = fill(d.target.index); return chColor; })\n");
		htmlBuilder.append("		.attr(\"d\", path);\n");
		htmlBuilder.append("		// Add an elaborate mouseover title for each chord.\n");
		htmlBuilder.append("		chord.append(\"title\").text(function(d) { return Chordnodes[d.source.index] + \" -> \" + Chordnodes[d.target.index] + \": \" + formatPercent(d.source.value) + \"\\n\" + Chordnodes[d.target.index] + \" -> \" + Chordnodes[d.source.index] + \": \" + formatPercent(d.target.value); });\n");
		htmlBuilder.append("		function mouseover(d, i) { chord.classed(\"fade\", function(p) { return p.source.index != i && p.target.index != i; }); }\n");
		htmlBuilder.append("	</script>\n");
		
		// add notes
		// 1. online plug-in. To offline, download and configure the js file
		// 2. many resources which makes many nodes with less than one pixel. We will fix this problem in future versions.
		// 
		htmlBuilder.append("<hr></hr>\n");
		htmlBuilder.append("<p><ul> <li>You need to be connected to internet to see the Chord Diagram. The diagram can be available offline by downloading d3.js and configuring it.</li> <li>The huge number of nodes can result in empty diagram since there will be less than a pixel to show each node. We will solve this problem in future versions.</li> </ul></p>\n");
		
		htmlBuilder.append("</body>\n");
		htmlBuilder.append("</html>\n");
		
		return htmlBuilder.toString();
	}
	
	public String getSocialNetworkMatrix(SocialNetwork sn){
		int size = sn.getSNNodes().size();
		double[][] matrix = new double[size][size];
		
		LinkedList nodes = new LinkedList();
		for(SNNode n : sn.getNodes()){
			nodes.add(n);
		}
		
		for(SNEdge e : sn.getSNEdges()){
			int i = nodes.indexOf(e.getSource());
			int j = nodes.indexOf(e.getTarget());
			
			matrix[i][j] += e.getWeight();
		}
		
		String matrixStr = "var matrix = [\n";
		for(int i=0; i<size; i++){
			matrixStr += "[";
			for(int j=0; j<size; j++){
				matrixStr += matrix[i][j] + ",";
			}
			matrixStr = matrixStr.substring(0, matrixStr.length()-1) + "],\n";
		}
		matrixStr = matrixStr.substring(0, matrixStr.length()-2) + "];\n";
		
		System.out.println(matrixStr);
		
		return matrixStr;
//		return "var matrix = [[0.0,0.0,0.125,0.0,0.0],[0.125,0.0,0.0,0.125,0.125],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.125,0.0,0.125],[0.0,0.0,0.125,0.125,0.0]];\n";
	}
	
	public String getSocialNetworkNodes(SocialNetwork sn){
		LinkedList nodes = new LinkedList();
		for(SNNode n : sn.getNodes()){
			nodes.add(n);
		}
		
		String nodesStr = "var Chordnodes = [";
		for(Object n : nodes){
			nodesStr += "\"" + ( (SNNode) n ).getLabel() + "\","; 
		}
		nodesStr = nodesStr.substring(0, nodesStr.length()-1) + "];\n";
		
		System.out.println(nodesStr);
		
		return nodesStr;
//		return "var caption = [\"Albert\",\"Amin\",\"Jack\",\"Peter\",\"Wil\"];\n";
	}
}
